@@ -84,7 +84,10 @@ print STDERR Dumper( 'html_form_engine_import_input: safe/user input hrs:', $use
       $data = $user_input_hr->{ $name } if $exists;
       }
 
-print STDERR " form iiiiiiiiiiiiiiiiiiiiiiiiiiii [$name] [$data] [$re] [$exists]\n";
+    next unless $exists;
+    next if ref( $data ); # strip objects, i.e. file uploads
+      
+# print STDERR " form iiiiiiiiiiiiiiiiiiiiiiiiiiii [$name] [$data] [$re] [$exists] {".ref($data)."}\n";
     
 #    next unless $exists;  # FIXME: should be an option
     
@@ -170,7 +173,7 @@ sub html_form_engine_display
     $text .= "<td align=right>$label</td>";
     $text .= "<td align=left>";
 
-print STDERR " form ffffffffffffffffffffffffffff [$name] [$value] [$re_help]\n";
+    # print STDERR " form ffffffffffffffffffffffffffff [$name] [$value] [$re_help]\n";
 
     if( $type =~ /^(STRING|STR|CHAR|TEXT|INPUT)$/ )
       {
@@ -188,9 +191,13 @@ print STDERR " form ffffffffffffffffffffffffffff [$name] [$value] [$re_help]\n";
       {
       $text .= $form->button( NAME => $name, VALUE => $value );
       }
+    elsif( $type =~ /^(FILE)$/ )
+      {
+      $text .= "<input type='file' name='$name' size='16'>";
+      }
     else
       {
-      confess "invalid form entry type\n"; # TODO: dump and function arguments
+      confess "invalid form entry type [$type]\n"; # TODO: dump and function arguments
       }
     $text .= "<span style='color: #f00'>$re_help</span>" if $error;
     $text .= "</td>";
